<?php

namespace NoccyLabs\BlockDevice;

use IteratorAggregate;
use ArrayAccess;
use ArrayIterator;

class Device implements IteratorAggregate, ArrayAccess
{
    /** @var string $name Device name */
    protected $name;
    /** @var Device[] $children Child devices */
    protected $children = [];
    /** @var string[] $props Device props */
    protected $props = [];

    public function __construct($name, array $props=null)
    {
        // Prepend /dev/ if we only get the basename, ex. sda1 -> /dev/sda1
        if (strpos($name,"/")===false) {
            $name = "/dev/{$name}";
        }
        $this->name = $name;

        // Set or read the properties
        if (is_array($props) && count($props)) {
            $this->initProps($props);
        } else {
            $this->readProps($name);
        }
    }

    protected function initProps(array $props)
    {
        if (array_key_exists("children", $props)) {
            $children = $props['children'];
            unset($props['children']);
            foreach ($children as $child) {
                $this->children[] = new Device($child['name'], $child);
            }
        }
        $this->props = $props;
    }

    protected function readProps($name)
    {
        exec("lsblk -JOpb {$name}", $output, $retval);
        if ($retval !== 0) {
            throw new DeviceListException("Unable to query device information");            
        }

        $json = join("",$output);
        $props = json_decode($json, true);
        if (!array_key_exists("blockdevices", $props)) {
            throw new DeviceListException("Invalid data returned from lsblk");
        }

        $deviceprops = reset($props['blockdevices']);

        $this->initProps($deviceprops);
    }

    public function getIterator()
    {
        return new ArrayIterator($this->props);
    }

    public function offsetSet($key,$value)
    {
    }

    public function offsetGet($key)
    {
        return array_key_exists($key, $this->props)
            ?$this->props[$key]
            :null;
    }

    public function offsetExists($key)
    {
        return array_key_exists($key, $this->props);
    }

    public function offsetUnset($key)
    {
    }

    public function getChildren()
    {
        return $this->children;
    }

    public function asString()
    {
        return sprintf("%s (%s)", $this->props['name'], $this->props['fstype']);
    }

}

