<?php

namespace NoccyLabs\BlockDevice;

class DeviceList
{
    /** @var Device[] $devices The devices in the list */
    protected $devices = [];

    public function __construct()
    {
        $this->readDevices();
    }

    public function readDevices()
    {
        exec("lsblk -JOpb", $output, $retval);
        if ($retval !== 0) {
            throw new DeviceListException("Unable to query device information");            
        }

        $json = join("",$output);
        $props = json_decode($json, true);
        if (!array_key_exists("blockdevices", $props)) {
            throw new DeviceListException("Invalid data returned from lsblk");
        }

        $this->devices = [];

        $devices = $props['blockdevices'];
        foreach ($devices as $props) {
            $this->devices[$props['name']] = new Device($props['name'], $props);
        }
    }

    public function getDevices()
    {
        return $this->devices;
    }

    public function getAllDevices()
    {
        $ret = [];
        $stack = $this->devices;
        while (count($stack)>0) {
            $device = array_shift($stack);
            $ret[] = $device;
            if (($children = $device->getChildren())) {
                $stack = array_merge($children, $stack);
            }
        }
        return $ret;
    }

    public function getDevice($name)
    {
        $stack = $this->devices;
        while (count($stack)>0) {
            $device = array_shift($stack);
            if ($device['name'] == $name) {
                return $device;
            }
            if (($children = $device->getChildren())) {
                $stack = array_merge($stack, $children);
            }
        }
        return null;
    }

}

